"""Main."""
import warnings
from typing import Union, List, Any
from types import ModuleType


def call_single(loading_data: dict, search_modules: List[ModuleType], **kwargs) -> Any:
    """From the list of modules, find the class written in yaml and instantiate it."""
    assert len(loading_data.keys()) == 1, "len(loading_data.keys()) != 1"
    if isinstance(loading_data, dict):
        for instance_or_function in _call_instance_or_function(
            loading_data, search_modules, **kwargs
        ):
            return instance_or_function
    else:
        TypeError


def call_multiple(
    loading_data: Union[dict, List[dict]], search_modules: List[ModuleType], **kwargs
) -> list:
    """From the list of modules, find the class written in yaml and instantiate it."""
    instance_list = []
    if isinstance(loading_data, list):
        for conf in loading_data:
            for instance in _call_instance_or_function(conf, search_modules, **kwargs):
                instance_list.append(instance)
    elif isinstance(loading_data, dict):
        for instance in _call_instance_or_function(
            loading_data, search_modules, **kwargs
        ):
            instance_list.append(instance)
    else:
        TypeError

    return instance_list


def yaml2instances(
    loading_data: Union[dict, List[dict]], search_modules: List[ModuleType], **kwargs
) -> list:
    """From the list of modules, find the class written in yaml and instantiate it."""
    warnings.warn("yaml2instances is deprecated. Use call_multiple_process.")
    instance_list = []
    if isinstance(loading_data, list):
        for conf in loading_data:
            for instance in _call_instance_or_function(conf, search_modules, **kwargs):
                instance_list.append(instance)
    elif isinstance(loading_data, dict):
        for instance in _call_instance_or_function(
            loading_data, search_modules, **kwargs
        ):
            instance_list.append(instance)
    else:
        TypeError

    return instance_list


def _call_instance_or_function(
    loading_data: dict, search_modules: List[ModuleType], **kwargs
):
    modules_dict = {}
    for module in search_modules:
        _modules = module.__dict__
        modules_dict.update(_modules)
    for _item, _kwargs in loading_data.items():
        base = {}
        base.update(kwargs or {})
        base.update(_kwargs or {})
        instance = modules_dict[_item](**base)
        yield instance



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['yaml2instance']

package_data = \
{'': ['*']}

install_requires = \
['black>=19.10b0,<20.0',
 'flake8>=3.8.1,<4.0.0',
 'mypy>=0.770,<0.771',
 'pytest>=5.4.2,<6.0.0',
 'pyyaml>=5.3.1,<6.0.0']

setup_kwargs = {
    'name': 'yaml2instance',
    'version': '0.2.0',
    'description': 'Find the class written in yaml and instantiate it.',
    'long_description': None,
    'author': 'shuns0314',
    'author_email': 'shunsuke.naka0314@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
